from pages.models import Page, Content
from rest_framework import serializers
from django.contrib.auth import get_user_model
from pages import settings
from django.contrib.sites.models import Site


class ContentSerializer(serializers.ModelSerializer):
    class Meta:
        model = Content
        fields = '__all__'


class PageSerializer(serializers.ModelSerializer):
    content_set = ContentSerializer(many=True, read_only=True)
    creation_date = serializers.DateTimeField()
    uuid = serializers.UUIDField()

    class Meta:
        model = Page
        fields = '__all__'

    def create(self, validated_data):

        attributes = (
            'status', 'delegate_to', 'freeze_date', 'creation_date',
            'publication_end_date', 'template', 'redirect_to_url',
            'last_modification_date', 'publication_date', 'uuid')

        admin = get_user_model().objects.filter(is_superuser=True)[0]

        cleaned_data = {}
        for attribute in attributes:
            cleaned_data[attribute] = validated_data.get(attribute)

        if validated_data.get('parent', False):
            cleaned_data['parent'] = validated_data.get('parent')

        cleaned_data['author'] = admin

        page = Page.objects.create(**cleaned_data)
        if settings.PAGE_USE_SITE_ID:
            site = Site.objects.get_current()
            page.sites.add(site)

        page.invalidate()
        return page



#!/usr/bin/env python3
import argparse
import os
import shutil
import stat
from subprocess import call
import fileinput
import random
PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
EXAMPLE_DIR = os.path.join(PROJECT_DIR, 'example')

parser = argparse.ArgumentParser(description='Gerbi CMS console tool.')
parser.add_argument('--create', type=str,
                    help='Create a new CMS example website')


def print_green(msg):
    print('\033[92m' + msg + '\033[0m')
            
def secret_key():
  return ''.join(
    [random.SystemRandom().choice(
      'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)]
  )

def new_secret_key_settings(filename):
  for line in fileinput.input(filename, inplace=True, backup='.bak'):
      if line.startswith('SECRET_KEY ='):
          print("SECRET_KEY = '{}'".format(secret_key()))
      else:
          print(line, end='')
  os.remove(filename + '.bak')

def main():
    args = parser.parse_args()

    if args.create:
        dirname = args.create
        absolute = os.path.join(os.getcwd(), dirname)
        print_green("Creating website {}".format(dirname))
        print_green("Copying example files to {}".format(dirname))
        ignore = shutil.ignore_patterns('*.db', '*.pyc', 'media', 'whoosh*')
        shutil.copytree(EXAMPLE_DIR, dirname, ignore=ignore)
        st = os.stat(os.path.join(dirname, 'manage.py'))
        os.chmod(os.path.join(dirname, 'manage.py'), st.st_mode | stat.S_IEXEC)
        new_secret_key_settings(os.path.join(absolute, 'settings.py'))
        ret = call(['./manage.py', 'migrate'], cwd=absolute)
        if ret != 0:
            return
        print_green('Migration done')
        _input = input("Would you like to create a superuser to connect to the admin? [N/y] ")
        if _input.lower() == 'y':
            call(['./manage.py', 'createsuperuser'], cwd=absolute)
        print_green('Creating demo pages')
        call(['./manage.py', 'pages_demo'], cwd=absolute)
        print_green('Rebuild search index')
        call(['./manage.py', 'rebuild_index', '--noinput'], cwd=absolute)
        print_green('Run webserver')
        call(['./manage.py', 'runserver'], cwd=absolute)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()


from pages.checks import page_templates_loading_check

from django.test import TestCase
from django.core.checks import Warning
from django.template import TemplateSyntaxError


class PageTemplatesLoadingCheckTestCase(TestCase):
    def test_check_detects_unexistant_template(self):
        unexistant = ('does_not_exists.html', 'foo')
        with self.settings(PAGE_TEMPLATES=[unexistant]):
            errors = page_templates_loading_check([])

        self.assertEqual(errors, [Warning(
            'Django cannot find template does_not_exists.html',
            obj=unexistant, id='pages.W001')])

    def test_check_doesnt_warn_on_existing_templates(self):
        with self.settings(PAGE_TEMPLATES=[('pages/contact.html', 'bas')]):
            errors = page_templates_loading_check([])

        self.assertEqual(errors, [])

    def test_template_syntax_error_is_not_silenced(self):
        with self.settings(PAGE_TEMPLATES=[('syntax_error.html', 'fail')]):
            with self.assertRaises(TemplateSyntaxError):
                page_templates_loading_check([])



# -*- coding: utf-8 -*-
"""Page Admin module."""
from pages import settings
from pages.models import Page, Content, PageAlias, Media
from pages.phttp import get_language_from_request, get_template_from_request
from pages.utils import get_placeholders
from pages.templatetags.pages_tags import PlaceholderNode
from pages.admin.forms import make_form
from pages.admin import views
from pages.urlconf_registry import registry

from os.path import join
import urllib
from collections import defaultdict
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import force_text
from django.conf import settings as global_settings
from django.http import HttpResponseRedirect, Http404
from django.contrib.admin.sites import AlreadyRegistered

extra_actions_registery = []
def add_page_action(action):
    if action not in extra_actions_registery:
        extra_actions_registery.append(action)


class PageAdmin(admin.ModelAdmin):
    """Page Admin class."""

    actions = extra_actions_registery

    # these mandatory fields are not versioned
    mandatory_placeholders = ('title', 'slug')
    general_fields = [
        'title', 'slug', 'status', 'target',
        'position', 'freeze_date', 'template', 'language',
        'redirect_to', 'redirect_to_url']

    if settings.PAGE_USE_SITE_ID and not settings.PAGE_HIDE_SITES:
        general_fields.append('sites')
    insert_point = general_fields.index('status') + 1

    # Strange django behavior. If not provided, django will try to find
    # 'page' foreign key in all registered models
    inlines = []

    if settings.PAGE_TAGGING:
        general_fields.insert(insert_point, 'tags')

    # Add support for future dating and expiration based on settings.
    if settings.PAGE_SHOW_END_DATE:
        general_fields.insert(insert_point, 'publication_end_date')
    if settings.PAGE_SHOW_START_DATE:
        general_fields.insert(insert_point, 'publication_date')

    general_fields.append('delegate_to')
    insert_point = general_fields.index('status') + 1

    page_templates = settings.get_page_templates()

    fieldsets = (
        [_('General'), {
            'fields': general_fields,
            'classes': ('module-general',),
        }],
    )

    class Media:
        css = {
            'all': [join(settings.PAGES_STATIC_URL, path) for path in (
                'css/rte.css',
                'css/pages.css'
            )]
        }
        js = [join(settings.PAGES_STATIC_URL, path) for path in (
            'javascript/jquery.js',
            'javascript/jquery.rte.js',
            'javascript/pages.js',
            'javascript/pages_list.js',
            'javascript/pages_form.js',
            'javascript/jquery.query-2.1.7.js',
            'javascript/iframeResizer.min.js',
        )]

    def get_urls(self):
        urls = super(PageAdmin, self).get_urls()
        from django.conf.urls import url

        pages_urls = [
            url(r'^$', self.list_pages, name='page-changelist'),
            url(r'^(?P<page_id>[0-9]+)/traduction/(?P<language_id>[-\w]+)/$',
                views.traduction, name='page-traduction'),
            url(r'^(?P<page_id>[0-9]+)/get-content/(?P<content_id>[0-9]+)/$',
                views.get_content, name='page-get-content'),
            url(r'^(?P<page_id>[0-9]+)/modify-content/(?P<content_type>[-\w]+)/(?P<language_id>[-\w]+)/$',
                views.modify_content, name='page-modify-content'),
            url(r'^(?P<page_id>[0-9]+)/modify-placeholder/$',
                views.modify_placeholder, name='page-modify-placeholder'),
            url(r'^(?P<page_id>[0-9]+)/get-last-content/$',
                views.get_last_content, name='page-get-last-content'),
            url(r'^(?P<page_id>[0-9]+)/delete-content/(?P<language_id>[-\w]+)/$',
                views.delete_content, name='page-delete-content'),
            url(r'^(?P<page_id>[0-9]+)/sub-menu/$',
                views.sub_menu, name='page-sub-menu'),
            url(r'^(?P<page_id>[0-9]+)/move-page/$',
                views.move_page, name='page-move-page'),
            url(r'^(?P<page_id>[0-9]+)/change-status/$',
                views.change_status, name='page-change-status'),
            url(r'^(?P<media_id>[0-9]+)/media-url/$',
                views.get_media_url, name='get-media-url'),
        ]

        return pages_urls + urls

    def save_model(self, request, page, form, change):
        """Move the page in the tree if necessary and save every
        placeholder :class:`Content <pages.models.Content>`.
        """
        language = form.cleaned_data['language']
        target = form.data.get('target', None)
        position = form.data.get('position', None)
        page.save()

        # if True, we need to move the page
        if target and position:
            try:
                target = self.model.objects.get(pk=target)
            except self.model.DoesNotExist:
                pass
            else:
                target.invalidate()
                page.move_to(target, position)

        for name in self.mandatory_placeholders:
            data = form.cleaned_data[name]
            placeholder = PlaceholderNode(name)
            extra_data = placeholder.get_extra_data(form.data)
            placeholder.save(page, language, data, change,
                extra_data=extra_data)

        for placeholder in get_placeholders(page.get_template()):
            if(placeholder.ctype in form.cleaned_data and placeholder.ctype
                    not in self.mandatory_placeholders):
                data = form.cleaned_data[placeholder.ctype]
                extra_data = placeholder.get_extra_data(form.data)
                placeholder.save(page, language, data, change,
                    extra_data=extra_data)

        page.invalidate()

    def get_fieldsets(self, request, obj=None):
        """
        Add fieldsets of placeholders to the list of already
        existing fieldsets.
        """

        # some ugly business to remove freeze_date
        # from the field list
        general_module = {
            'fields': list(self.general_fields),
            'classes': ('module-general',),
        }

        default_fieldsets = list(self.fieldsets)
        if not request.user.has_perm('pages.can_freeze'):
            general_module['fields'].remove('freeze_date')
        if not request.user.has_perm('pages.can_publish'):
            general_module['fields'].remove('status')
        if not len(registry):
            general_module['fields'].remove('delegate_to')

        default_fieldsets[0][1] = general_module

        placeholder_fieldsets = []
        section_placeholder_fieldsets = defaultdict(list)
        template = get_template_from_request(request, obj)
        for placeholder in get_placeholders(template):
            if placeholder.ctype not in self.mandatory_placeholders and not placeholder.section:
                placeholder_fieldsets.append(placeholder.ctype)
            elif placeholder.section:
                section_placeholder_fieldsets[placeholder.section].append(placeholder.ctype)

        additional_fieldsets = []
        for title, fieldset in section_placeholder_fieldsets.items():
            additional_fieldsets.append((_(title), {
                'fields': fieldset,
                'classes': ('module-content collapse grp-collapse grp-closed',),
            }))
        additional_fieldsets.append((_('Content'), {
            'fields': placeholder_fieldsets,
            'classes': ('module-content',),
        }))
        return default_fieldsets + additional_fieldsets

    def save_form(self, request, form, change):
        """Given a ModelForm return an unsaved instance. ``change`` is True if
        the object is being changed, and False if it's being added."""
        instance = super(PageAdmin, self).save_form(request, form, change)
        instance.template = form.cleaned_data['template']
        if not change:
            instance.author = request.user
        return instance

    def get_form(self, request, obj=None, **kwargs):
        """Get a :class:`Page <pages.admin.forms.PageForm>` for the
        :class:`Page <pages.models.Page>` and modify its fields depending on
        the request."""

        template = get_template_from_request(request, obj)

        #model = create_page_model(get_placeholders(template))

        form = make_form(self.model, get_placeholders(template))

        # bound the form
        language = get_language_from_request(request)
        form.base_fields['language'].initial = language
        if obj:
            initial_slug = obj.slug(language=language, fallback=False)
            initial_title = obj.title(language=language, fallback=False)
            form.base_fields['slug'].initial = initial_slug
            form.base_fields['title'].initial = initial_title

        template = get_template_from_request(request, obj)
        page_templates = settings.get_page_templates()
        template_choices = list(page_templates)
        # is default template is not in the list add it
        if not [tpl for tpl in template_choices if tpl[0] == settings.PAGE_DEFAULT_TEMPLATE]:
            template_choices.insert(0, (settings.PAGE_DEFAULT_TEMPLATE,
                    _('Default template')))
        form.base_fields['template'].choices = template_choices
        form.base_fields['template'].initial = force_text(template)

        for placeholder in get_placeholders(template):
            ctype = placeholder.ctype
            if obj:
                initial = placeholder.get_content(obj, language, lang_fallback=False)
            else:
                initial = None
            form.base_fields[ctype] = placeholder.get_field(obj,
                language, initial=initial)

        return form

    def change_view(self, request, object_id, form_url='', extra_context=None):
        """The ``change`` admin view for the
        :class:`Page <pages.models.Page>`."""
        language = get_language_from_request(request)
        extra_context = {
            'language': language,
            'page_languages': settings.PAGE_LANGUAGES,
        }
        try:
            int(object_id)
        except ValueError:
            raise Http404('The "%s" part of the location is invalid.'
                % str(object_id))
        try:
            obj = self.model.objects.get(pk=object_id)
        except self.model.DoesNotExist:
            # Don't raise Http404 just yet, because we haven't checked
            # permissions yet. We don't want an unauthenticated user to be able
            # to determine whether a given object exists.
            obj = None
        else:
            template = get_template_from_request(request, obj)
            extra_context['placeholders'] = get_placeholders(template)
            extra_context['traduction_languages'] = [l for l in
                settings.PAGE_LANGUAGES if Content.objects.get_content(obj,
                                    l[0], "title") and l[0] != language]
        extra_context['page'] = obj

        response = super(PageAdmin, self).change_view(request, object_id,
            form_url=form_url, extra_context=extra_context)
        if request.method == 'POST' and isinstance(response, HttpResponseRedirect):
            if '_continue' in request.POST or '_saveasnew' in request.POST or '_addanother' in request.POST:
                addlanguage = True
            else:
                addlanguage = False
            if addlanguage:
                splitted = list(urllib.parse.urlparse(response.url))
                query = urllib.parse.parse_qs(splitted[4])
                query['language'] = language
                splitted[4] = urllib.parse.urlencode(query)
                response = HttpResponseRedirect(urllib.parse.urlunparse(splitted))
        return response

    def add_view(self, request, form_url='', extra_context=None):
        """The ``add`` admin view for the :class:`Page <pages.models.Page>`."""
        extra_context = {
            'language': get_language_from_request(request),
            'page_languages': settings.PAGE_LANGUAGES,
        }
        return super(PageAdmin, self).add_view(request, form_url,
                                               extra_context)

    def has_add_permission(self, request):
        """Return ``True`` if the current user has permission to add a new
        page."""
        return request.user.has_perm('pages.add_page')

    def has_change_permission(self, request, obj=None):
        """Return ``True`` if the current user has permission
        to change the page."""
        return request.user.has_perm('pages.change_page')

    def has_delete_permission(self, request, obj=None):
        """Return ``True`` if the current user has permission on the page."""
        return request.user.has_perm('pages.delete_page')

    def list_pages(self, request, template_name=None, extra_context=None):
        """List root pages"""
        if not self.admin_site.has_permission(request):
            return self.admin_site.login(request)
        language = get_language_from_request(request)

        query = request.POST.get('q', '').strip()

        if query:
            page_ids = list(set([c.page.pk for c in
                Content.objects.filter(body__icontains=query)]))
            pages = Page.objects.filter(pk__in=page_ids)
        else:
            pages = Page.objects.root()
        if settings.PAGE_HIDE_SITES:
            pages = pages.filter(sites=global_settings.SITE_ID)

        context = {
            'can_publish': request.user.has_perm('pages.can_publish'),
            'can_import': settings.PAGE_IMPORT_ENABLED,
            'lang': language,  # lang is the key used by show_content
            'pages': pages,
            'opts': self.model._meta,
            'q': query
        }

        context.update(extra_context or {})
        change_list = self.changelist_view(request, context)

        return change_list


try:
    admin.site.register(Page, PageAdmin)
except AlreadyRegistered:
    pass

class ContentAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'type', 'language', 'page')
    list_filter = ('page',)
    search_fields = ('body',)


class AliasAdmin(admin.ModelAdmin):
    list_display = ('page', 'url',)
    list_editable = ('url',)
    raw_id_fields = ['page']

try:
    admin.site.register(PageAlias, AliasAdmin)
except AlreadyRegistered:
    pass


class MediaAdmin(admin.ModelAdmin):
    list_display = ('image', 'title', 'creation_date', 'description', 'extension')
    list_display_links = ('image', 'title', )

try:
    admin.site.register(Media, MediaAdmin)
except AlreadyRegistered:
    pass



from django.core.management.base import BaseCommand, CommandError
from pages.models import Page
from pages.tests.testcase import new_page
from django.contrib.auth import get_user_model


lorem = """Lorem ipsum dolor sit amet, consectetur adipiscing elit. 
Quisque tempus tellus enim, quis tempus dui pretium non. 
Cras eget enim vel magna fringilla cursus ut sit amet mi. 
Curabitur id pharetra turpis. Pellentesque quis eros nunc. 
Etiam interdum nisi ut sapien facilisis ornare. Mauris in tellus elit. 
Integer vulputate venenatis odio. Vivamus in diam vitae magna gravida 
sodales sit amet id ex. Aliquam commodo massa at mollis blandit. 
Donec auctor sapien et risus gravida, ultrices imperdiet est laoreet."""


class Command(BaseCommand):
    help = 'Create a couple of dummy pages for a demo'

    def handle(self, *args, **options):

        UserModel = get_user_model()
        if UserModel.objects.count() == 0:
            UserModel.objects.create(username='demo', password='demo')

        new_page(content={
            'title': 'Home', 'slug': 'home', 'lead': 'Welcome to the Gerbi CMS', 'content': lorem
        }, template='index.html')
        prod_page = new_page(content={
            'title': 'Products', 'slug': 'products', 'lead': 'Our products', 'content': lorem
        }, template='index.html')
        new_page(content={
            'title': 'Poney', 'slug': 'poney', 'lead': 'Our shiny poney', 'content': lorem},
            parent=prod_page,
            template='index.html')
        new_page(content={
            'title': 'Hat', 'slug': 'hat', 'lead': 'Fedora for elegance', 'content': lorem},
            parent=prod_page,
            template='index.html')
        blog = new_page(content={
            'title': 'Blog', 'slug': 'blog', 'lead': 'Blog example', 'content': lorem},
            template='blog-home.html', delegate_to='blog')
        new_page(content={
            'title': 'Blog post 1', 'slug': 'blog-post-1', 'lead': 'Blog post example', 'content': lorem},
            parent=blog,
            template='blog-post.html')
        new_page(content={
            'title': 'Blog post 2', 'slug': 'blog-post-2', 'lead': 'Blog post example', 'content': lorem},
            parent=blog,
            template='blog-post.html')
        new_page(content={
            'title': 'Blog post 3', 'slug': 'blog-post-3', 'lead': 'Blog post example', 'content': lorem},
            parent=blog,
            template='blog-post.html')
        new_page(content={
            'title': 'Blog post 4', 'slug': 'blog-post-4', 'lead': 'Blog post example', 'content': lorem},
            parent=blog,
            template='blog-post.html')
        new_page(content={
            'title': 'Contact', 'slug': 'contact', 
            'lead': 'Contact us at gerbi@example.com', 'content':lorem
        }, template='index.html')
        



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        ('pages', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='sites',
            field=models.ManyToManyField(default=[1], help_text='The site(s) the page is accessible at.', verbose_name='sites', to='sites.Site'),
        ),
    ]



# -*- coding: utf-8 -*-
"""Django page CMS command tests suite module."""
from pages.models import Page, Content, PageAlias
from pages.tests.testcase import TestCase
from django.core.management import call_command
from django.test import LiveServerTestCase
import json

from django.test.testcases import LiveServerThread, QuietWSGIRequestHandler
from django.core.servers.basehttp import WSGIServer

class LiveServerSingleThread(LiveServerThread):
    """Runs a single threaded server rather than multi threaded. Reverts https://github.com/django/django/pull/7832"""

    def _create_server(self):
        return WSGIServer((self.host, self.port), QuietWSGIRequestHandler, allow_reuse_address=False)


class LiveServerSingleThreadedTestCase(LiveServerTestCase):
    "A thin sub-class which only sets the single-threaded server as a class"
    server_thread_class = LiveServerSingleThread


class CommandTestCase(TestCase, LiveServerSingleThreadedTestCase):
    """Django page CMS command tests suite class."""

    def test_pull(self):
        """Pull command get the correct data"""
        self.new_page(content={'title': 'pull-page', 'slug': 'pull-slug'})
        url =  self.live_server_url + '/pages/api/'
        filename = '/tmp/test'
        call_command('pages_pull', 'admin:b', filename=filename, host=url, verbosity=0)
        with open(filename, "r") as f:
            data = f.read()
            pages = json.loads(data)
            for content in pages[0]['content_set']:
                 self.assertTrue(content['body'] in ['pull-page', 'pull-slug'])

    def test_push(self):
        """Push command put back the content properly"""
        url =  self.live_server_url + '/pages/api/'
        page1 = self.new_page(content={'title': 'pull-page', 'slug': 'pull-slug'})
        page2 = self.new_page(content={'title': 'pull-page-2', 'slug': 'pull-slug-2'})
        call_command('pages_pull', 'admin:b', filename='/tmp/test', host=url, verbosity=0)
        page1.delete()
        self.assertEqual(Page.objects.all().count(), 1)
        call_command('pages_push', 'admin:b', filename='/tmp/test', host=url, verbosity=0)
        self.assertEqual(Page.objects.all().count(), 2)

    def test_tree(self):
        """Push command" restore the tree properly"""
        url =  self.live_server_url + '/pages/api/'
        filename = '/tmp/test'
        page1 = self.new_page(content={'title': 'pull-page-1', 'slug': 'pull-slug-1'})
        page2 = self.new_page(content={'title': 'pull-page-2', 'slug': 'pull-slug-2'}, parent=page1)
        page3 = self.new_page(content={'title': 'pull-page-3', 'slug': 'pull-slug-3'}, parent=page2)

        self.assertSequenceEqual(page1.get_children(), [page2])
        self.assertSequenceEqual(page2.get_children(), [page3])

        call_command('pages_pull', 'admin:b', filename=filename, host=url, verbosity=0)
        page2.move_to(page1, 'left')

        self.assertSequenceEqual(page1.get_children(), [])
        self.assertSequenceEqual(page2.get_children(), [page3])

        call_command('pages_push', 'admin:b', filename='/tmp/test', host=url, verbosity=0)

        self.assertSequenceEqual(page1.get_children(), [page2])
        self.assertSequenceEqual(page2.get_children(), [page3])

    def test_tree_delete(self):
        """Push command tree delete"""
        url =  self.live_server_url + '/pages/api/'
        filename = '/tmp/test'
        page1 = self.new_page(content={'title': 'pull-page-1', 'slug': 'pull-slug-1'})
        page2 = self.new_page(content={'title': 'pull-page-2', 'slug': 'pull-slug-2'}, parent=page1)
        page3 = self.new_page(content={'title': 'pull-page-3', 'slug': 'pull-slug-3'}, parent=page2)

        self.assertSequenceEqual(page1.get_children(), [page2])
        self.assertSequenceEqual(page2.get_children(), [page3])


        call_command('pages_pull', 'admin:b', filename=filename, host=url, verbosity=0)

        page4 = self.new_page(content={'title': 'pull-page-4', 'slug': 'pull-slug-4'}, parent=page1)
        self.assertSequenceEqual(page1.get_children(), [page2, page4])
        page2.delete()
        self.assertSequenceEqual(page1.get_children(), [page4])

        call_command('pages_push', 'admin:b', filename='/tmp/test', host=url, verbosity=0)

        page2 = Page.objects.from_slug('pull-slug-2')
        self.assertSequenceEqual(page1.get_children(), [page4, page2])




# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-01-19 06:28
from __future__ import unicode_literals

from django.db import migrations, models
import pages.models
import pages.utils


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0005_media'),
    ]

    operations = [
        migrations.AddField(
            model_name='media',
            name='creation_date',
            field=models.DateTimeField(default=pages.utils.get_now, editable=False, verbose_name='creation date'),
        ),
        migrations.AlterField(
            model_name='media',
            name='url',
            field=models.FileField(upload_to=pages.models.media_filename),
        ),
    ]



from django.core.management.base import BaseCommand, CommandError
import requests
import os
import json
from pages.management.utils import APICommand

class Command(APICommand):
    help = 'Pull data from a Django Page CMS API'

    def handle(self, *args, **options):
        self.parse_options(options)

        self.cprint("Fetching page data on " + self.host)
        self.host = self.host + '?format=json'

        page_list = requests.get(self.host, auth=self.auth, timeout=5)
        if page_list.status_code != 200:
            self.http_error(response)
        data = json.loads(page_list.text)
        
        if not os.path.exists(os.path.dirname(self.filename)):
            os.makedirs(os.path.dirname(self.filename))
        with open(self.filename, "w") as f:
            f.write(json.dumps(data))
        self.cprint(self.filename + " written to disk")



# Generated by Django 2.1.3 on 2018-11-02 08:18

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0007_language_code'),
    ]

    operations = [
        migrations.AlterField(
            model_name='media',
            name='extension',
            field=models.CharField(blank=True, editable=False, max_length=32),
        ),
        migrations.AlterField(
            model_name='page',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='pages.Page', verbose_name='parent'),
        ),
        migrations.AlterField(
            model_name='page',
            name='redirect_to',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='redirected_pages', to='pages.Page', verbose_name='redirect to'),
        ),
        migrations.AlterField(
            model_name='page',
            name='redirect_to_url',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='redirect to url'),
        ),
        migrations.AlterField(
            model_name='pagealias',
            name='page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='pages.Page', verbose_name='page'),
        ),
    ]



#!/usr/bin/env python
import os, sys
current_dirname = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(current_dirname, '..'))
sys.path.insert(0, os.path.join(current_dirname, '../..'))

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "test_settings")

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



from django.conf.urls import url
from blog import views
from django.urls import include, path, re_path

urlpatterns = [
  url(r'^category/(?P<tag_id>[0-9]+)$', views.category_view, name='blog_category_view'),
  url(r'^$', views.blog_index, name='blog_index')
]


# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-01-19 05:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0004_auto_20161209_0648'),
    ]

    operations = [
        migrations.CreateModel(
            name='Media',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=255)),
                ('description', models.TextField(blank=True)),
                ('url', models.FileField(upload_to='')),
                ('extension', models.CharField(blank=True, max_length=32)),
            ],
            options={
                'verbose_name_plural': 'Medias',
            },
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class BasicCmsConfig(AppConfig):
    name = 'pages'
    verbose_name = _("Pages")

    def ready(self):
        from . import checks



from django.db import models
from django.forms import ModelForm
from pages.models import Page
from django.utils.translation import ugettext_lazy as _


class Document(models.Model):
    "A dummy model used to illustrate the use of linked models in django-page-cms"

    title = models.CharField(_('title'), max_length=100, blank=False)
    text = models.TextField(_('text'), blank=True)

    # the foreign key _must_ be called page
    page = models.ForeignKey(Page, on_delete=models.CASCADE)


class DocumentForm(ModelForm):
    class Meta:
        model = Document
        fields = "__all__"



#!/usr/bin/env python3
"""Django's command-line utility for administrative tasks."""
import os
import sys

example_dir = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(example_dir, '..'))

def main():
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "settings")
    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    execute_from_command_line(sys.argv)


if __name__ == '__main__':
    main()




from django.conf import settings
from django.conf.urls import url
from django.urls import include, path, re_path
from django.conf.urls.static import static
from django.contrib import admin
from pages import views

from pages.urlconf_registry import register_urlconf
register_urlconf('blog', 'blog.urls', label='Blog index')

urlpatterns = [
    url(r'^i18n/', include('django.conf.urls.i18n')),
    url(r'^admin/', admin.site.urls),
]

try:
	import haystack
	urlpatterns += [url(r'^search/', include('haystack.urls'), name='haystack_search')]
except ImportError:
	pass

urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
urlpatterns += static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)

urlpatterns += [
    url(r'^$', views.details, {'path': '', 'name': 'pages-root'}),
    url(r'^', include('pages.urls')),
]

admin.site.site_header = 'Gerbi Admin'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0006_auto_20170119_0628'),
    ]

    operations = [
        migrations.AlterField(
            model_name='content',
            name='language',
            field=models.CharField(max_length=7, verbose_name='language'),
        ),
    ]



# -*- coding: utf-8 -*-
"""Pages admin views"""
from pages import settings
from pages.models import Page, Content, Media
from pages.utils import get_placeholders
from pages.phttp import get_language_from_request

from django.shortcuts import get_object_or_404, render
from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.contrib.admin.views.decorators import staff_member_required
from django.template import RequestContext
from django import template
from django.views.decorators.csrf import csrf_exempt
from django.urls import reverse
from django import forms
from pages.managers import fake_page


@staff_member_required
@csrf_exempt
def change_status(request, page_id):
    """
    Switch the status of a page.
    """
    perm = request.user.has_perm('pages.change_page')
    if perm and request.method == 'POST':
        page = Page.objects.get(pk=page_id)
        page.status = int(request.POST['status'])
        page.invalidate()
        page.save()
        return HttpResponse(str(page.status))
    raise Http404


@staff_member_required
def list_pages_ajax(request, invalid_move=False):
    """Render pages table for ajax function."""
    language = get_language_from_request(request)
    pages = Page.objects.root()
    context = {
        'can_publish': request.user.has_perm('pages.can_publish'),
        'invalid_move': invalid_move,
        'language': language,
        'pages': pages,
    }
    return render(
        request, "admin/pages/page/change_list_table.html",
        context)


@staff_member_required
@csrf_exempt
def modify_content(request, page_id, content_type, language_id):
    """Modify the content of a page."""
    page = get_object_or_404(Page, pk=page_id)
    perm = request.user.has_perm('pages.change_page')
    if perm and request.method == 'POST':
        content = request.POST.get('content', False)
        if not content:
            raise Http404
        if settings.PAGE_CONTENT_REVISION:
            Content.objects.create_content_if_changed(page, language_id,
                                                      content_type, content)
        else:
            Content.objects.set_or_create_content(page, language_id,
                                                  content_type, content)
        if page:
            page.invalidate()
            # to update last modification date
            page.save()
        else:
            fake_page.invalidate(content_type)

        return HttpResponse('ok')
    raise Http404


@staff_member_required
@csrf_exempt
def modify_placeholder(request, page_id):
    """Modify the content of a page."""
    content_type = request.GET.get('content_type')
    language_id = request.GET.get('language_id')
    page = get_object_or_404(Page, pk=page_id)
    perm = request.user.has_perm('pages.change_page')
    if perm and request.method == 'POST':
        placeholders = get_placeholders(page.get_template())
        for placeholder in placeholders:
            if placeholder.name == content_type:
                initial = placeholder.get_content(
                    page, language_id, lang_fallback=False)
                form = forms.Form(request.POST, request.FILES)
                form.fields[content_type] = placeholder.get_field(
                    page, language_id, initial=initial)
                if not form.is_valid():
                    return HttpResponse(form.as_p())

                if placeholder.shared:
                    save_page = None
                else:
                    save_page = page

                extra_data = placeholder.get_extra_data(form.data)

                placeholder.save(
                    save_page, language_id,
                    form.cleaned_data[content_type], True, extra_data)
                if save_page:
                    page.invalidate()
                    # to update last modification date
                    page.save()
                else:
                    fake_page.invalidate(content_type)

                return HttpResponse('ok')
        raise Http404("Content type not found in placeholders")

    raise Http404


@staff_member_required
def get_last_content(request, page_id):
    """Get the latest content for a particular type"""
    content_type = request.GET.get('content_type')
    language_id = request.GET.get('language_id')
    page = get_object_or_404(Page, pk=page_id)
    placeholders = get_placeholders(page.get_template())
    _template = template.loader.get_template(page.get_template())
    for placeholder in placeholders:
        if placeholder.name == content_type:
            context = RequestContext(request, {
                'current_page': page,
                'lang': language_id
            })
            with context.bind_template(_template.template):
                content = placeholder.render(context)
                return HttpResponse(content)
    raise Http404


@staff_member_required
def delete_content(request, page_id, language_id):
    page = get_object_or_404(Page, pk=page_id)
    perm = request.user.has_perm('pages.delete_page')
    if not perm:
        raise Http404

    for c in Content.objects.filter(page=page, language=language_id):
        c.delete()

    destination = reverse("admin:pages_page_change", args=[page_id])
    return HttpResponseRedirect(destination)


@staff_member_required
def traduction(request, page_id, language_id):
    """Traduction helper."""
    page = Page.objects.get(pk=page_id)
    lang = language_id
    placeholders = get_placeholders(page.get_template())
    language_error = (
        Content.objects.get_content(page, language_id, "title")
        is None
    )
    return render(request, 'pages/traduction_helper.html', {
        'page': page,
        'lang': lang,
        'language_error': language_error,
        'placeholders': placeholders,
    })


@staff_member_required
def get_content(request, page_id, content_id):
    """Get the content for a particular page"""
    content = Content.objects.get(pk=content_id)
    return HttpResponse(content.body)


@staff_member_required
@csrf_exempt
def move_page(request, page_id, extra_context=None):
    """Move the page to the requested target, at the given
    position."""
    page = Page.objects.get(pk=page_id)

    target = request.POST.get('target', None)
    position = request.POST.get('position', None)
    if target is not None and position is not None:
        try:
            target = Page.objects.get(pk=target)
        except Page.DoesNotExist:
            pass
            # TODO: should use the django message system
            # to display this message
            # _('Page could not been moved.')
        else:
            # move_to invalidates cache on a model level
            from mptt.exceptions import InvalidMove
            invalid_move = False
            try:
                page.move_to(target, position)
            except InvalidMove:
                invalid_move = True
            return list_pages_ajax(request, invalid_move)
    return HttpResponseRedirect('../../')


@staff_member_required
def sub_menu(request, page_id):
    """Render the children of the requested page with the sub_menu
    template."""
    page = Page.objects.get(id=page_id)
    pages = page.children.all()
    page_languages = settings.PAGE_LANGUAGES
    return render(request, "admin/pages/page/sub_menu.html", {
        'can_publish': request.user.has_perm('pages.can_publish'),
        'page': page,
        'pages': pages,
        'page_languages': page_languages,
    })

import os
@staff_member_required
def get_media_url(request, media_id):
    """Get media URL."""
    media = Media.objects.get(id=media_id)
    return HttpResponse(media.url.name)



# -*- coding: utf-8 -*-
"""Django CMS come with a set of ready to use widgets that you can enable
in the admin via a placeholder tag in your template."""

from pages.settings import PAGES_MEDIA_URL, PAGES_STATIC_URL
from pages.settings import PAGE_LANGUAGES
from pages.models import Page
from pages.widgets_registry import register_widget

from django import forms
from django.forms import TextInput, Textarea
from django.forms import MultiWidget
from django.forms import FileInput as DFileInput
from django.contrib.admin.widgets import AdminTextInputWidget
from django.contrib.admin.widgets import AdminTextareaWidget
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from os.path import join

register_widget(TextInput)
register_widget(Textarea)
register_widget(AdminTextInputWidget)
register_widget(AdminTextareaWidget)


class RichTextarea(Textarea):
    """A RichTextarea widget."""
    class Media:
        js = [join(PAGES_STATIC_URL, path) for path in (
            'javascript/jquery.js',
            'javascript/jquery.rte.js'
        )]
        css = {
            'all': [join(PAGES_STATIC_URL, path) for path in (
                'css/rte.css',
                'css/font-awesome.min.css'
            )]
        }

    def __init__(self, language=None, attrs=None, **kwargs):
        attrs = {'class': 'rte'}
        self.language = language
        super(RichTextarea, self).__init__(attrs)

    def render(self, name, value, attrs=None, **kwargs):
        rendered = super(RichTextarea, self).render(name, value, attrs)
        context = {
            'name': name,
            'PAGES_STATIC_URL': PAGES_STATIC_URL,
            'PAGES_MEDIA_URL': PAGES_MEDIA_URL,
        }
        return rendered + mark_safe(render_to_string(
            'pages/widgets/richtextarea.html', context))


register_widget(RichTextarea)


insert_image_link = u'''
<br>
<button title='insert image from the media library' class='image-lookup-{name}'>
    From media library
</button>
<input name="{name}-selected" id="{name}-selected" type="hidden">
<span id="{name}-selected-value">
</span>

<br><label for="{name}-delete">
<input name="{name}-delete" style="display:inline-block" id="{name}-delete" type="checkbox" value="true"> {del_msg}
</label>
<br style="clear:both">


<script>
$(function(){{
    function dismissRelatedLookupPopup(win, chosenId) {{
        $.get('/admin/pages/page/' + chosenId + '/media-url/', function(response) {{
            console.log(response);
            $('#{name}-selected').val(response);
            $('#{name}-selected-value').text(response);
        }});
        win.close();
        window.dismissRelatedLookupPopup = oldDismissRelatedLookupPopup;
        window.dismissAddRelatedObjectPopup = oldDismissAddRelatedObjectPopup;
    }}
    function showMediaAdminPopup() {{
        var name = 'mediaWindowSelect';
        var href = '/admin/pages/media/?_to_field=id&_popup=1';
        window.dismissRelatedLookupPopup = dismissRelatedLookupPopup;
        window.dismissAddRelatedObjectPopup = dismissRelatedLookupPopup;
        var win = window.open(href, name, 'height=500,width=800,resizable=yes,scrollbars=yes');
        win.focus();
        return false;
    }}
    $('.image-lookup-{name}').click(function(e) {{
        e.preventDefault();
        showMediaAdminPopup();
        return false;
    }});
}});

</script>
'''

class FileInput(DFileInput):

    def __init__(self, page=None, language=None, attrs=None, **kwargs):
        self.language = language
        self.page = page
        super(FileInput, self).__init__(attrs)

    please_save_msg = _('Please save the page to show the file field')
    delete_msg = _('Delete file')

    def render(self, name, value, attrs=None, **kwargs):
        if not self.page:
            field_content = self.please_save_msg
        else:
            field_content = '<span class="placeholder-fileinput">'
            if value:
                field_content += _('Current file: %s<br/>') % value
                field_content += '<hr>'
            field_content += super(FileInput, self).render(name, attrs)
            field_content += insert_image_link.format(
                name=name,
                del_msg=self.delete_msg,
                value=value)
            field_content += '</span>'
        return mark_safe(field_content)
register_widget(FileInput)


class ImageInput(FileInput):

    please_save_msg = _('Please save the page to show the image field')
    delete_msg = _('Delete image')

register_widget(ImageInput)


class LanguageChoiceWidget(TextInput):

    def __init__(self, language=None, attrs=None, **kwargs):
        self.language = language
        self.page = kwargs.get('page')
        # page is None
        super(LanguageChoiceWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None, **kwargs):
        context = {
            'name': name,
            'value': value,
            'page': self.page,
            'language': value,
            'page_languages': PAGE_LANGUAGES
        }
        return mark_safe(render_to_string(
            'pages/widgets/languages.html', context))


class PageLinkWidget(MultiWidget):
    '''A page link `Widget` for the admin.'''

    def __init__(
            self, attrs=None, page=None, language=None,
            video_url=None, linkedpage=None, text=None):
        l = [('', '----')]
        for p in Page.objects.all():
            l.append((p.id, str(p)))
        widgets = [
            forms.Select(choices=l),
            TextInput(attrs=attrs)
        ]
        super(PageLinkWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        import json
        try:
            return json.loads(value)
        except:
            pass
        return []

    def value_from_datadict(self, data, files, name):
        import json
        value = ['', '']
        for da in [x for x in data if x.startswith(name)]:
            index = int(da[len(name) + 1:])
            value[index] = data[da]
        if value[0] == value[1] == '':
            return None
        return json.dumps(value)

    def _has_changed(self, initial, data):
        """Need to be reimplemented to be correct."""
        if data == initial:
            return False
        return bool(initial) != bool(data)

    def format_output(self, rendered_widgets):
        """
        Given a list of rendered widgets (as strings), it inserts an HTML
        linebreak between them.

        Returns a Unicode string representing the HTML for the whole lot.
        """
        return """<table>
            <tr><td>page</td><td>%s</td></tr>
            <tr><td>text</td><td>%s</td></tr>
        </table>""" % tuple(rendered_widgets)
register_widget(PageLinkWidget)



from django.shortcuts import render
from pages.models import Page
from taggit.models import Tag
from django.core.paginator import Paginator

def category_view(request, *args, **kwargs):
    context = dict(kwargs)
    category = Tag.objects.get(id=kwargs['tag_id'])
    page = context['current_page']
    blogs = page.get_children_for_frontend().filter(tags__name__in=[category.name])

    paginator = Paginator(blogs, 8)
    page_index = request.GET.get('page')
    blogs = paginator.get_page(page_index)
    context['blogs'] = blogs
    
    context['category'] = category.name
    return render(request, 'blog-home.html', context)

def blog_index(request, *args, **kwargs):
    context = dict(kwargs)
    page = context['current_page']
    blogs = page.get_children_for_frontend()

    paginator = Paginator(blogs, 7)
    page = request.GET.get('page')
    blogs = paginator.get_page(page)
    context['blogs'] = blogs
    context['template_name'] = 'blog-home.html'

    return render(request, 'blog-home.html', context)


# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-12-09 06:48
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0003_page_uuid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='content',
            name='page',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='pages.Page', verbose_name='page'),
        ),
    ]



# -*- coding: utf-8 -*-
"""Django page CMS module."""
VERSION = (2, 0, 12)
__version__ = '.'.join(map(str, VERSION))
__author__ = "Batiste Bieler"
__contact__ = "batiste.bieler@gmail.com"
__homepage__ = "https://github.com/batiste/django-page-cms"
__docformat__ = "restructuredtext"
__doc__ = 'A tree based Django CMS application'
__license__ = 'BSD'
__keywords__ = ['django', 'cms']

default_app_config = 'pages.app_config.BasicCmsConfig'




import pages.admin
from pages.plugins.jsonexport.actions import export_pages_as_json
from pages.plugins.jsonexport.actions import import_pages_from_json
pages.admin.add_page_action(export_pages_as_json)
pages.admin.add_page_action(import_pages_from_json)


# # -*- coding: utf-8 -*-
# """Django page CMS selemium test module"""
# from unittest import SkipTest

# from django.contrib import auth
# from django.urls import reverse
# from django.test import LiveServerTestCase
# from pages import settings
# from pages.models import Page
# from pages.tests.testcase import TestCase
# from selenium import webdriver
# from selenium.common.exceptions import TimeoutException
# from selenium.webdriver.support.ui import WebDriverWait

# screenshot_nb = 1
# TIMEOUT = 10


# class SeleniumTestCase(TestCase, LiveServerTestCase):

#     def setUp(self):
#         self.browser = webdriver.PhantomJS()
#         self.browser.set_page_load_timeout(TIMEOUT)
#         self.get_admin_client()

#         # setUp is where you instantiate the selenium webdriver and loads the browser.
#         auth.models.User.objects.create_superuser(
#             username='admin_s',
#             password='admin',
#             email='admin_s@example.com'
#         )

#         self.browser.get('%s%s' % (self.live_server_url, reverse("admin:index")))

#         super(SeleniumTestCase, self).setUp()

#     def screenshot(self):
#         global screenshot_nb
#         if settings.PAGE_TESTS_SAVE_SCREENSHOTS:
#             self.browser.save_screenshot('screenshot_%d.png' % screenshot_nb)
#             screenshot_nb += 1

#     def select_option(self, select, option_id):
#         for option in select.find_elements_by_tag_name('option'):
#             if option.get_attribute('value') == str(option_id):
#                 option.click()

#     def visit(self, url):
#         # Open the django admin page.
#         # DjangoLiveServerTestCase provides a live server url attribute
#         # to access the base url in tests
#         url = '%s%s' % (self.live_server_url, url)
#         try:
#             return self.browser.get(url)
#         except TimeoutException:
#             raise SkipTest("Timeout: get({0})".format(repr(url)))

#     def find_element_by_css_selector(self, selector):
#         return self.timeout('find_element_by_css_selector', selector)

#     def find_elements_by_css_selector(self, selector):
#         return self.timeout('find_elements_by_css_selector', selector)

#     def find_element_by_id(self, id):
#         return self.timeout('find_element_by_id', id)

#     def timeout(self, command, param, timeout=TIMEOUT):
#         wait = WebDriverWait(self.browser, timeout)
#         try:
#             return wait.until(lambda b: getattr(b, command)(param))
#         except TimeoutException:
#             raise SkipTest("Timeout: {0}({1})".format(command, repr(param)))

#     def click(self, selector):
#         return self.browser.find_element_by_css_selector(selector).click()

#     def login(self):
#         self.visit(reverse("admin:index"))
#         # Fill login information of admin
#         username = self.find_element_by_id("id_username")
#         username.send_keys("admin_s")
#         password = self.find_element_by_id("id_password")
#         password.send_keys("admin")
#         self.click("input[type='submit']")

#     def tearDown(self):
#         self.browser.close()
#         try:
#             self.browser.quit()
#         except OSError:
#             # http://stackoverflow.com/questions/42705674/python-selenium-phantomjs-quit-error
#             pass
#         super(SeleniumTestCase, self).tearDown()

#     def url_change(self, id):
#         return reverse('admin:pages_page_change', args=[id])

#     def test_admin_select(self):
#         self.login()
#         page = self.new_page()
#         self.visit(self.url_change(page.id))
#         status = self.find_element_by_id('id_status')
#         self.assertEqual(status.get_attribute('value'), str(page.status))

#         self.select_option(status, str(Page.DRAFT))
#         self.assertEqual(status.get_attribute('value'), str(Page.DRAFT))

#         src = self.find_element_by_css_selector('.field-status img [src$="draft.gif"]')

#     def test_admin_move_page(self):
#         self.login()
#         page_1 = self.new_page({'slug': 'p1'})
#         page_2 = self.new_page({'slug': 'p2'})
#         self.visit(reverse('admin:pages_page_changelist'))

#         rows = self.find_elements_by_css_selector('#page-list tbody tr')
#         row_1 = rows[0]
#         row_2 = rows[1]

#         self.assertEqual(row_1.get_attribute('id'), 'page-row-%d' % page_1.id)
#         self.assertEqual(row_2.get_attribute('id'), 'page-row-%d' % page_2.id)

#         page_3 = self.new_page({'slug': 'p3'})

#         self.click('#move-link-%d' % page_2.id)
#         self.click('#move-target-%d .move-target.left' % page_1.id)
#         self.visit(reverse('admin:pages_page_changelist'))

#         self.find_element_by_id('page-row-%d' % page_3.id)

#     def test_admin_export_json(self):
#         self.login()
#         self.new_page({'slug': 'p1'})
#         self.new_page({'slug': 'p2'})
#         self.visit(reverse('admin:pages_page_changelist'))

#         self.find_elements_by_css_selector('#action-toggle')[0].click()

#         action_select = self.find_elements_by_css_selector(
#             '[name="action"]')[0]
#         self.select_option(action_select, 'export_pages_as_json')

#         self.find_elements_by_css_selector('[name="index"]')[0].click()

#         # apparently there is no easy way to test a download?



"""Django page CMS urlconf registry."""
from django.utils.translation import ugettext as _


class UrlconfAlreadyRegistered(Exception):
    """
    An attempt was made to register a urlconf for Django page CMS more
    than once.
    """


class UrlconfNotFound(Exception):
    """
    The requested urlconf was not found
    """


registry = []


def get_choices():
    choices = [('', 'No delegation')]
    for reg in registry:
        if reg[2]:
            label = reg[2]
        else:
            label = reg[0]
        choices.append((reg[0], label))
    return choices


def register_urlconf(name, urlconf, label=None):
    for urlconf_tuple in registry:
        if urlconf_tuple[0] == name:
            raise UrlconfAlreadyRegistered(
                _('The urlconf %s has already been registered.') % name)
    urlconf_tuple = (name, urlconf, label, urlconf)
    registry.append(urlconf_tuple)


def get_urlconf(name):
    for urlconf_tuple in registry:
        if urlconf_tuple[0] == name:
            return urlconf_tuple[1]
    raise UrlconfNotFound(
        _('The urlconf %s has not been registered.') % name)
